@@ -17,7 +17,7 @@ public:
 	RectC llBounds() {return _bounds;}
 
 	int zoom() const {return _zi;}
-	void setZoom(int zoom) {_zi = zoom;}
+	void setZoom(int zoom);
 	int zoomFit(const QSize &size, const RectC &rect);
 	int zoomIn();
 	int zoomOut();
@@ -27,7 +27,7 @@ public:
 	Coordinates xy2ll(const QPointF &p);
 
 	void load(const Projection &in, const Projection &out, qreal deviceRatio,
-	  bool hidpi, int style, int layer);
+	  bool hidpi, bool hillShading, int style, int layer);
 	void unload();
 
 	void draw(QPainter *painter, const QRectF &rect, Flags flags);
@@ -58,7 +58,6 @@ private:
 		  {return level < other.level;}
 	};
 
-	QRect rect(const Zoom &zoom) const;
 	bool readHeader(QDataStream &stream);
 	bool readRegions(QDataStream &stream);
 	bool computeBounds();
@@ -66,6 +65,8 @@ private:
 	QByteArray tileData(const QPoint &tile);
 	void drawTile(QPainter *painter, QPixmap &pixmap, QPointF &tp);
 
+	static QRect rect(const Zoom &zoom);
+
 	friend QDebug operator<<(QDebug dbg, const Region &region);
 	friend QDebug operator<<(QDebug dbg, const Zoom &zoom);
 
@@ -76,6 +77,8 @@ private:
 	int _tileSize;
 	QList<Zoom> _zooms;
 
+	qreal _factor;
+
 	bool _valid;
 	QString _errorString;
 };
